@@ -1,5 +1,6 @@
 import os
 import re
+import io
 import json
 import logging
 import collections
@@ -43,7 +44,8 @@ from .utils import (
     logout_from_server,
     create_entity_id,
     entity_data_json_default,
-    failed_json_default
+    failed_json_default,
+    TransferProgress,
 )
 
 JSONDecodeError = getattr(json, "JSONDecodeError", ValueError)
@@ -200,6 +202,15 @@ class ServerAPIBase(object):
 
         self._thumbnail_cache = ThumbnailCache(True)
 
+    def get_base_url(self):
+        return self._base_url
+
+    def get_rest_url(self):
+        return self._rest_url
+
+    base_url = property(get_base_url)
+    rest_url = property(get_rest_url)
+
     @property
     def access_token(self):
         return self._access_token
@@ -544,22 +555,125 @@ class ServerAPIBase(object):
     def delete(self, entrypoint, **kwargs):
         return self.raw_delete(entrypoint, params=kwargs)
 
-    def download_file(self, endpoint, filepath, chunk_size=None):
-        if not chunk_size:
-            # 1 MB chunk by default
-            chunk_size = 1024 * 1024
+    def _download_file(self, url, filepath, chunk_size, progress):
         dst_directory = os.path.dirname(filepath)
         if not os.path.exists(dst_directory):
             os.makedirs(dst_directory)
 
+        kwargs = {"stream": True}
+        if self._session is None:
+            kwargs["headers"] = self.get_headers()
+            get_func = self._base_functions_mapping[RequestTypes.get]
+        else:
+            get_func = self._session_functions_mapping[RequestTypes.get]
+
         with open(filepath, "wb") as f_stream:
-            with self.raw_get(endpoint, stream=True) as response:
+            with get_func(url, **kwargs) as response:
+                response.raise_for_status()
+                progress.set_content_size(response.headers["Content-length"])
                 for chunk in response.iter_content(chunk_size=chunk_size):
                     f_stream.write(chunk)
+                    progress.add_downloaded_chunk(len(chunk))
+
+    def download_file(self, endpoint, filepath, chunk_size=None, progress=None):
+        """Download file from AYON server.
+
+        Endpoint can be full url (must start with 'base_url' of api object).
+
+        Progress object can be used to track download. Can be used when
+        download happens in thread and other thread want to catch changes over
+        time.
+
+        Args:
+            endpoint (str): Endpoint or URL to file that should be downloaded.
+            filepath (str): Path where file will be downloaded.
+            chunk_size (int): Size of chunks that are received in single loop.
+            progress (TransferProgress): Object that gives ability to track
+                download progress.
+        """
+
+        if not chunk_size:
+            # 1 MB chunk by default
+            chunk_size = 1024 * 1024
+
+        if endpoint.startswith(self._base_url):
+            url = endpoint
+        else:
+            endpoint = endpoint.lstrip("/").rstrip("/")
+            url = "{}/{}".format(self._rest_url, endpoint)
+
+        # Create dummy object so the function does not have to check
+        #   'progress' variable everywhere
+        if progress is None:
+            progress = TransferProgress()
+
+        progress.set_source_url(url)
+        progress.set_destination_url(filepath)
+        progress.set_started()
+        try:
+            self._download_file(url, filepath, chunk_size, progress)
+
+        except Exception as exc:
+            progress.set_failed(str(exc))
+            raise
+
+        finally:
+            progress.set_done()
+
+    def _upload_file(self, url, filepath, progress):
+        kwargs = {}
+        if self._session is None:
+            kwargs["headers"] = self.get_headers()
+            post_func = self._base_functions_mapping[RequestTypes.post]
+        else:
+            post_func = self._session_functions_mapping[RequestTypes.post]
 
-    def upload_file(self, endpoint, filepath):
         with open(filepath, "rb") as stream:
-            self.raw_post(endpoint, data=stream)
+            stream.seek(0, io.SEEK_END)
+            size = stream.tell()
+            stream.seek(0)
+            progress.set_content_size(size)
+            response = post_func(url, data=stream, **kwargs)
+        response.raise_for_status()
+        progress.set_downloaded_size(size)
+
+    def upload_file(self, endpoint, filepath, progress=None):
+        """Upload file to server.
+
+        Todos:
+            Uploading with more detailed progress.
+
+        Args:
+            endpoint (str): Endpoint or url where file will be uploaded.
+            filepath (str): Source filepath.
+            progress (TransferProgress): Object that gives ability to track
+                upload progress.
+        """
+
+        if endpoint.startswith(self._base_url):
+            url = endpoint
+        else:
+            endpoint = endpoint.lstrip("/").rstrip("/")
+            url = "{}/{}".format(self._rest_url, endpoint)
+
+        # Create dummy object so the function does not have to check
+        #   'progress' variable everywhere
+        if progress is None:
+            progress = TransferProgress()
+
+        progress.set_source_url(filepath)
+        progress.set_destination_url(url)
+        progress.set_started()
+
+        try:
+            self._upload_file(url, filepath, progress)
+
+        except Exception as exc:
+            progress.set_failed(str(exc))
+            raise
+
+        finally:
+            progress.set_done()
 
     def trigger_server_restart(self):
         result = self.post("system/restart")
@@ -779,6 +893,55 @@ class ServerAPIBase(object):
             raise ServerError(response.text)
         return response.data
 
+    def download_addon_private_file(
+        self,
+        addon_name,
+        addon_version,
+        filename,
+        destination_dir,
+        destination_filename=None,
+        chunk_size=None,
+        progress=None,
+    ):
+        """Download a file from addon private files.
+
+        This method requires to have authorized token available. Private files
+        are not under '/api' restpoint.
+
+        Args:
+            addon_name (str): Addon name.
+            addon_version (str): Addon version.
+            filename (str): Filename in private folder on server.
+            destination_dir (str): Where the file should be downloaded.
+            destination_filename (str): Name of destination filename. Source
+                filename is used if not passed.
+            chunk_size (int): Download chunk size.
+            progress (TransferProgress): Object that gives ability to track
+                download progress.
+
+        Returns:
+            str: Filepath to downloaded file.
+        """
+
+        if not destination_filename:
+            destination_filename = filename
+        dst_filepath = os.path.join(destination_dir, destination_filename)
+        # Filename can contain "subfolders"
+        dst_dirpath = os.path.dirname(dst_filepath)
+        if not os.path.exists(dst_dirpath):
+            os.makedirs(dst_dirpath)
+
+        url = "{}/addons/{}/{}/private/{}".format(
+            self._base_url,
+            addon_name,
+            addon_version,
+            filename
+        )
+        self.download_file(
+            url, dst_filepath, chunk_size=chunk_size, progress=progress
+        )
+        return dst_filepath
+
     def get_dependencies_info(self):
         result = self.get("dependencies")
         return result.data
@@ -839,29 +1002,53 @@ class ServerAPIBase(object):
 
     def download_dependency_package(
         self,
-        dst_directory,
         package_name,
-        platform_name=None
+        dst_directory,
+        filename,
+        platform_name=None,
+        chunk_size=None,
+        progress=None,
     ):
+        """Download dependency package from server.
+
+        This method requires to have authorized token available. The package is
+        only downloaded.
+
+        Args:
+            package_name (str): Name of package to download.
+            dst_directory (str): Where the file should be downloaded.
+            filename (str): Name of destination filename.
+            platform_name (str): Name of platform for which the dependency
+                package is targetter. Default value is current platform.
+            chunk_size (int): Download chunk size.
+            progress (TransferProgress): Object that gives ability to track
+                download progress.
+
+        Returns:
+            str: Filepath to downloaded file.
+       """
         if platform_name is None:
             platform_name = platform.system().lower()
 
-        package_filepath = os.path.join(dst_directory, package_name + ".zip")
+        package_filepath = os.path.join(dst_directory, filename)
         self.download_file(
             "dependencies/{}/{}".format(package_name, platform_name),
-            package_filepath
+            package_filepath,
+            chunk_size=chunk_size,
+            progress=progress
         )
         return package_filepath
 
     def upload_dependency_package(
-        self, filepath, package_name, platform_name=None
+        self, filepath, package_name, platform_name=None, progress=None
     ):
         if platform_name is None:
             platform_name = platform.system().lower()
 
         self.upload_file(
             "dependencies/{}/{}".format(package_name, platform_name),
-            filepath
+            filepath,
+            progress=progress
         )
 
     def delete_dependency_package(self, package_name, platform_name=None):
